
#include "stdafx.h"
#include "algs.h"

#include <iomanip>
#include <fstream>
#include <math.h>

using namespace std;

struct ELEMENT_INFO {
	int r;
    bool selected;
	ELEMENT_INFO() {r = 0; selected = false;}
};

////////////////////////////////////////////////////////////////////////
/////        

void connectivity(CMatrix &R, std::vector<ELEMENT_INFO> &conn_ty) {
	int r;
	for (int i = 0; i < R.Width(); ++i) {
		r = 0;
		for (int j = 0; j < R.Height(); ++j)
			r += R(i, j);
		conn_ty[i].r = r;
	}
}

void zero_line(intvect &vect) {
	for (int i = 0; i < vect.size(); ++i)
		vect[i] = 0;
}

int line_sum(const intvect &vect) {
    int n = 0;
	for (int v = 0; v < vect.size(); ++v)
		n += vect[v];
	return n;
}

void line_or(CMatrix &Q, int n, const intvect &vect, intvect &result) {
	for (int i = 0; i < Q.Width(); ++i)
		if (vect[i] == 0 && Q(n, i) == 0)
			result[i] = 0;
		else
			result[i] = 1;
}

void line_and(CMatrix &Q, int n, const intvect &vect, intvect &result) {
	for (int i = 0; i < Q.Width(); ++i)
		if (vect[i] == 1 && Q(n, i) == 1)
			result[i] = 1;
		else
			result[i] = 0;
}

void line_and(intvect &vect1, intvect &vect2, intvect &result) {
	for (int i = 0; i < vect1.size(); ++i)
		if (vect1[i] == 1 && vect2[i] == 1)
			result[i] = 1;
		else
			result[i] = 0;
}

void print_line(std::ofstream &log, const intvect &line, char *name, bool print_sum = false) {
	log << name;

	for (int i = 0; i < line.size(); ++i)
		log << line[i];

	if (print_sum)
		log << "|" << setw(3) << line_sum(line);

	log << endl;
}

void print_line(std::ofstream &log, CMatrix &Q, int n, char *name) {
	log << name;

	for (int i = 0; i < Q.Width(); ++i)
		log << (int)Q(n, i);

	log << endl;
}

void print_conn(std::ofstream &log, std::vector<ELEMENT_INFO> &conn_ty) {

	log << " ID:        ";

	for (int i = 1; i < conn_ty.size(); ++i)
		log << setw(3) << i;

	log << endl << " : ";

	for (int i = 1; i < conn_ty.size(); ++i)
		log << setw(3) << conn_ty[i].r;

	log << endl << " :    ";

	for (int i = 1; i < conn_ty.size(); ++i)
		log << setw(3) << (conn_ty[i].selected? "1": "");

	log << endl << endl;
}

int select_appropriate(std::ofstream &log, CMatrix &Q, const std::vector<ELEMENT_INFO> &conn_ty, 
					   const intvect &group, const intvect &group_connectors) {

	//      

	log << "*    ,  \n";

	intvect op_result(Q.Width());
	int bind_measure = 0;
	intvect max_bound;
	int n;

	for (int i = 1; i < conn_ty.size(); ++i)
		if (!conn_ty[i].selected) {

			log << "  " << i << endl;

			print_line(log, Q, i, "  :  ");
			print_line(log, group_connectors, "  :     ");

			line_and(Q, i, group_connectors, op_result);

			print_line(log, op_result, "  .: ", true);
			log <<  endl;

			int sum = line_sum(op_result);
			
			if (sum > bind_measure) {
				max_bound.resize(1);
				max_bound[0] = i;
				bind_measure = sum;
			}
			else if (sum == bind_measure) {
				int last = max_bound.size();
				max_bound.resize(last + 1);
				max_bound[last] = i;
			}
		}

		if (bind_measure == 0 || max_bound.size() == 0)
			return -1;

		int appropriate = -1;
		int min_external = INT_MAX;
		//  ,      

		log << "\n* ,      \n";

		for (int i = 0; i < max_bound.size(); ++i) {
			//   ,    
			zero_line(op_result);
			line_or(Q, max_bound[i], group_connectors, op_result);

			log << "\n       " << max_bound[i] << endl;
			print_line(log, op_result, "  :      ");

			//  
			intvect rest_connectors(Q.Width());
			for (int j = 1; j < Q.Height(); ++j) {
				bool skip = j == max_bound[i];
				if (!skip)
					for (int k = 0; k < group.size(); ++k)
						if (j == group[k]) {
							skip = true;
							break;
						}
				if (skip) continue;
				line_or(Q, j, rest_connectors, rest_connectors);
			}

			print_line(log, rest_connectors, "  : ");

			//   
			line_and(rest_connectors, op_result, op_result);
			print_line(log, op_result, "  :   ", true);

			int external = line_sum(op_result);
			if (min_external > external) {
				min_external = external;
				appropriate = max_bound[i];
			}
		}

		log << "\n  : " << appropriate << endl << endl;

	return appropriate;
}

void compose(CMatrix &Q, CMatrix &R, intvect2 &groups, int n_size) {
	
	std::ofstream log("compose.log");
	std::vector<ELEMENT_INFO> conn_ty(R.Height());
	
	groups.clear();

	//     
	connectivity(R, conn_ty);
	
	int composed = 0; //   

	//    ,   
	while (composed < conn_ty.size() - 1) {
		
		log << "=========   ========= \n\n"
			<< "**        \n";

		//      
		int max_conn = 1;

		//  . - -  
		while (conn_ty[max_conn].selected && max_conn < conn_ty.size()) ++max_conn;

		//  
		for (int i = max_conn; i < conn_ty.size(); ++i)
			if (!conn_ty[i].selected && conn_ty[i].r > conn_ty[max_conn].r)
				max_conn = i;

		conn_ty[max_conn].selected = true;

		//         
		int last_group = groups.size();
		groups.resize(last_group + 1);
		groups[last_group].resize(1);
		groups[last_group][0] = max_conn;
	
		composed++;

		print_conn(log, conn_ty);

		intvect group_connectors(Q.Width()); //   

		line_or(Q, max_conn, group_connectors, group_connectors);

		log << "  \n";
		print_line(log, group_connectors, "  ");
		log << endl;
		
		int next, size;
		while (groups[last_group].size() < n_size) {

			log << "\n**   \n";

			next = select_appropriate(log, Q, conn_ty, groups[last_group], group_connectors);
			if (next < 0)
				break;
			size = groups[last_group].size();
			groups[last_group].resize(size + 1);
			groups[last_group][size] = next;
			conn_ty[next].selected = true;
			line_or(Q, next, group_connectors, group_connectors);
			composed++;

			print_conn(log, conn_ty);
			log << "\n* \n";
			print_line(log, group_connectors, "  ");
			log << endl << endl;
		}
	}

}

////////////////////////////////////////////////////////////////////////
/////     

void print_positions(std::ofstream &log, intvect &placed) {

	log << setw(3) << placed[0] << setw(3) << placed[1] << setw(3) << placed[2] << endl
		<< setw(3) << placed[3] << setw(3) << placed[4] << setw(3) << placed[5] << endl
		<< setw(3) << placed[6] << setw(3) << placed[7] << setw(3) << placed[8] << endl;
}

void nonplaced_connectivity(CMatrix &R, std::vector<ELEMENT_INFO> &conn_ty) {

	int r;
	for (int i = 1; i < R.Height(); ++i) {
		if (!conn_ty[i].selected) {
			r = 0;
			for (int j = 1; j < R.Width(); ++j)
				if (conn_ty[j].selected)
					r += R(i, j);
			conn_ty[i].r = r;
		}
		else
			conn_ty[i].r = -1;
	}

}

int select_position(std::ofstream &log, CMatrix &R, CPoint pos[], 
					std::vector<ELEMENT_INFO> &conn_ty, intvect &placed, int r) {

	double F, F_min = INT_MAX;
	int target_pos = 0;

	for (int i = 0; i < placed.size(); ++i) {
		F = 0;
		if (placed[i] == 0) {

			//log << " : " << i << endl;

			for (int j = 1; j < R.Width(); ++j)
				if (conn_ty[j].selected) {

					//log << " : " << j << endl;
					int pos_index = -1;
					for (int l = 0; l < placed.size(); ++l)
						if (placed[l] == j) {
							pos_index = l;
							break;
						}

						//log << " : " << pos_index << endl << endl;

						int dx = pos[pos_index].x - pos[i].x;
						int dy = pos[pos_index].y - pos[i].y;
						F += R(r, j) * sqrt(double(dx*dx + dy*dy));
				}

			log /*<< endl*/ << "    (" << i / 3 << ", " << i % 3 << "): " << F << endl;

			if (F_min > F) {
				F_min = F;
				target_pos = i;
			}
		}
	}

	log << endl << " : (" << target_pos / 3 << ", " << target_pos % 3 << ")" << endl << endl;

	return target_pos;
}

void place(CMatrix &R, CPoint pos[], intvect &order) {
	
	std::ofstream log("place.log");
	std::vector<ELEMENT_INFO> conn_ty(R.Height());
	
	//     
	connectivity(R, conn_ty);

	for (int i = 0; i < order.size(); ++i)
		order[i] = 0;

	//    
	int max_conn = 1;
	for (int i = 1; i < conn_ty.size(); ++i)
		if (conn_ty[i].r > conn_ty[max_conn].r)
			max_conn = i;

	//   
	order[1] = max_conn;
	conn_ty[max_conn].selected = true;


	log << "**      \n";

	print_conn(log, conn_ty);


	for (int i = 0; i < order.size() - 1; ++i) {
		//  ,    
		nonplaced_connectivity(R, conn_ty);
		log << "\n**  \n";
		log << "    \n";		
		print_conn(log, conn_ty);
        
		max_conn = 1;
		for (int j = 1; j < conn_ty.size(); ++j)
			if (!conn_ty[j].selected && conn_ty[j].r > conn_ty[max_conn].r)
				max_conn = j;

		log << " : " << max_conn << endl << endl;
		
		order[
			select_position(log, R, pos, conn_ty, order, max_conn)
		] = max_conn;

		conn_ty[max_conn].selected = true;

		print_positions(log, order);
		log << endl;
	}

}

////////////////////////////////////////////////////////////////////////
/////  


void prepare(CMatrix &walls, CMatrix &work) {
	for (int i = 0; i < walls.Height(); ++i)
		for (int j = 0; j < walls.Width(); ++j)
			if (walls(i, j) != -1)
				work(i, j) = 0;
			else
				work(i, j) = -1;
}

void print_matrix(std::ofstream &log, CMatrix &mtx) {
	for (int i = 0; i < mtx.Height(); ++i) {
		for (int j = 0; j < (int)mtx.Width(); ++j)
			log << setw(3) << (int)mtx(i ,j);
		log << endl;
	}
	log.flush();

	log << endl << endl;
}

bool assign(CMatrix &field, int x, int y, int value) {
	
	if (x < 0 || x >= field.Width() || y < 0 || y >= field.Height())
		return true;

	if (field(x, y) == -2)
		return false;

	if (field(x, y) == 0) {
		field(x, y) = value;		
	}

	return true;
}

bool extract(CMatrix &field, std::vector<CPoint> &path, int round, int x, int y) {
	
	if (x < 0 || x >= field.Width() || y < 0 || y >= field.Height())
		return true;

	if (field(x, y) == round) {
		path[round - 1] = CPoint(x, y);
		return false;
	}

	return true;
}


int wave(CMatrix &field) {
	int round = 1;
	bool has_room = true;

	while (has_room) {
		has_room = false;
		for (int i = 0; i < field.Height(); ++i)
			for (int j = 0; j < field.Width(); ++j) 
				if (field(i, j) == round) {
					has_room = true;
					if (
						!(
							assign(field, i, j + 1, round + 1) &&
							assign(field, i, j - 1, round + 1) &&
							assign(field, i + 1, j, round + 1) &&
							assign(field, i - 1, j, round + 1)
						)
					)
						return round;
				}
		++round;
	}

	return -1;
}

void walk_back(CMatrix &field, std::vector<CPoint> &path) {
	int value;

	for (int i = path.size() - 1; i > 0; --i) {
		if (!extract(field, path, i, path[i].x + 1, path[i].y)) continue;
		if (!extract(field, path, i, path[i].x, path[i].y - 1)) continue;
		if (!extract(field, path, i, path[i].x - 1, path[i].y)) continue;
		extract(field, path, i, path[i].x, path[i].y + 1);
	}

}

void trace(connvect &connectors, CSize matrix_size, ptvect2 &result) {
	CMatrix walls(matrix_size.cx, matrix_size.cy);
	CMatrix work(matrix_size.cx, matrix_size.cy);

	std::ofstream log("trace.log");
	prepare(walls, walls);

	result.clear();

	//       
	connvect::iterator c = connectors.begin();
	for (; c != connectors.end(); ++c) {
		CPoint start = c->GetOutCell(0, 7, CPartBase::PIN_SPACE + 1);
		CPoint end = c->GetInCell(0, 7, CPartBase::PIN_SPACE + 1);

		walls(start.y - 1, start.x - 1) = -1;
		walls(end.y - 1, end.x - 1) = -1;
	}

	c = connectors.begin();
	for (; c != connectors.end(); ++c) {
		prepare(walls, work);

		CPoint start = c->GetOutCell(0, 7, CPartBase::PIN_SPACE + 1);
		CPoint end = c->GetInCell(0, 7, CPartBase::PIN_SPACE + 1);

		work(start.y - 1, start.x - 1) = 1;
		work(end.y - 1, end.x - 1) = -2;

		int steps = wave(work);

		int last = result.size();
		result.resize(last + 1);

		if (steps > 0) {
			result[last].resize(steps + 1);
			result[last][steps] = CPoint(end.y - 1, end.x - 1);

			print_matrix(log, work);

			walk_back(work, result[last]);
			
			for (int i = 0; i < result[last].size(); ++i) {
				walls(result[last][i].x,  result[last][i].y) = -1;
				
				long tmp = result[last][i].y;
				result[last][i].y = result[last][i].x + 1;
				result[last][i].x = tmp + 1;
				
				log << "(" << result[last][i].x << "," <<  result[last][i].y << ")";
			}

			log << endl << endl;
			print_matrix(log, walls);
		}
	}
}

